----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:14:18 04/08/2015 
-- Design Name: 
-- Module Name:    keyschedule - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity keyschedule_v2 is
	port (
		key_in	: in	std_logic_vector(79 downto 0);
		mode_i	: in std_logic;
		key_o	: out	std_logic_vector(79 downto 0)
	);
end keyschedule_v2;

architecture Behavioral of keyschedule_v2 is

	signal mixing_out_1, mixing_out_2, key_in_s : std_logic_vector(79 downto 0);

begin
	-- Inverse Permutation
    key_in_s(79 downto 60) <= key_in(63 downto 60) & key_in(79 downto 64);
    key_in_s(59 downto 40) <= key_in(43 downto 40) & key_in(59 downto 44);
    key_in_s(39 downto 20) <= key_in(23 downto 20) & key_in(39 downto 24);
    key_in_s(19 downto 0)  <=  key_in(3 downto 0) & key_in(19 downto 4);
	
	-- Encryption
	mixing_out_1(3 downto 0)  <= key_in(3 downto 0) xor (key_in(16) & key_in(19 downto 17));
	mixing_out_1(7 downto 4)  <= key_in(7 downto 4) xor ("000" & key_in(11));
	mixing_out_1(23 downto 8) <= key_in(23 downto 8);

	mixing_out_1(27 downto 24)  <= key_in(27 downto 24) xor (key_in(28) & "000");
	mixing_out_1(35 downto 28)  <= key_in(35 downto 28);
	mixing_out_1(39 downto 36)  <= key_in(39 downto 36) xor (key_in(34 downto 32) & key_in(35));
  
	mixing_out_1(43 downto 40) <= key_in(43 downto 40);
	mixing_out_1(47 downto 44) <= key_in(47 downto 44) xor (key_in(48) & key_in(51 downto 49));
	mixing_out_1(51 downto 48) <= key_in(51 downto 48);
	mixing_out_1(55 downto 52) <= key_in(55 downto 52) xor ("000" & key_in(51));
	mixing_out_1(59 downto 56) <= key_in(59 downto 56);
  
	mixing_out_1(63 downto 60) <= key_in(63 downto 60);
	mixing_out_1(67 downto 64) <= key_in(67 downto 64) xor (key_in(60) & "000") xor (key_in(70 downto 68) & key_in(71));
	mixing_out_1(79 downto 68) <= key_in(79 downto 68);
    
	-- Decryption
    mixing_out_2(3 downto 0)  <= key_in_s(3 downto 0) xor (key_in_s(16) & key_in_s(19 downto 17));
	mixing_out_2(7 downto 4)  <= key_in_s(7 downto 4) xor ("000" & key_in_s(11));
	mixing_out_2(23 downto 8) <= key_in_s(23 downto 8);

	mixing_out_2(27 downto 24)  <= key_in_s(27 downto 24) xor (key_in_s(28) & "000");
	mixing_out_2(35 downto 28)  <= key_in_s(35 downto 28);
	mixing_out_2(39 downto 36)  <= key_in_s(39 downto 36) xor (key_in_s(34 downto 32) & key_in_s(35));
  
	mixing_out_2(43 downto 40) <= key_in_s(43 downto 40);
	mixing_out_2(47 downto 44) <= key_in_s(47 downto 44) xor (key_in_s(48) & key_in_s(51 downto 49));
	mixing_out_2(51 downto 48) <= key_in_s(51 downto 48);
	mixing_out_2(55 downto 52) <= key_in_s(55 downto 52) xor ("000" & key_in_s(51));
	mixing_out_2(59 downto 56) <= key_in_s(59 downto 56);
  
	mixing_out_2(63 downto 60) <= key_in_s(63 downto 60);
	mixing_out_2(67 downto 64) <= key_in_s(67 downto 64) xor (key_in_s(60) & "000") xor (key_in_s(70 downto 68) & key_in_s(71));
	mixing_out_2(79 downto 68) <= key_in_s(79 downto 68);
		
	with mode_i select
    key_o(79 downto 60) <=
      mixing_out_2(79 downto 60) when '1',
      mixing_out_1(75 downto 60) & mixing_out_1(79 downto 76) when others;
      
  with mode_i select
    key_o(59 downto 40) <=
      mixing_out_2(59 downto 40) when '1',
      mixing_out_1(55 downto 40) & mixing_out_1(59 downto 56) when others;
      
  with mode_i select
    key_o(39 downto 20) <=
      mixing_out_2(39 downto 20) when '1',
      mixing_out_1(35 downto 20) & mixing_out_1(39 downto 36) when others;
      
  with mode_i select
    key_o(19 downto 0) <=
      mixing_out_2(19 downto 0) when '1',
      mixing_out_1(15 downto 0) & mixing_out_1(19 downto 16) when others;
	
end Behavioral;

